class IgConsentUi {
    constructor() {
        this.manager = window.IgConsentManager;
        this.consentHideStorageName = 'IgConsentManager_hide';
        this.manager.watch(this);
        this.elements = new Map();
        this.setElements();
        this.setButtons();
        this.isDisplayed();
    }

    getTemplate() {
        const Template = ({
            purpose,
            parent,
            serviceHtml,
            translate
        }) =>
            `
            <div class="${!parent ? 'purpose' : ''}">
                <div id="service-${purpose.name}" class="service-${purpose.name} cookieCategory checkbox-container">
                    <div class="${!parent ? 'purpose-header ' : ''}checkbox-headline flex">
                        <div class="flex${!parent ? ' headline-click' : ''}">
                            ${!parent ? '<div class="arrow"><svg xmlns="http://www.w3.org/2000/svg" xml:space="preserve" viewBox="0 0 16 16"><g fill="currentColor"><path d="m4.464 6.05-.707.707L8 11l4.243-4.243-.707-.707L8 9.586z"/></g></svg></div>' : ''}
                            <div class="checkbox-header">
                                <span class="cm-list-title">${purpose.title}</span>
                                ${purpose.required ? '<span class="cm-required">(' + translate['alwaysRequired'] + ')</span>' : ''}
                            </div>
                        </div>
                        <label for="service-check-${purpose.name}">
                            <input type="checkbox" id="service-check-${purpose.name}" name="cookieCategories" value="${purpose.name}" ${parent ? 'data-parent="' + parent + '"' : ''} ${purpose.required ? 'disabled="1"' : ''} ${purpose.required ? 'checked="1"' : ''} />
                            <div class="checkbox-slider"${purpose.required ? ' data-disabled="1"' : ''}></div>
                        </label>
                    </div>
                    <div class="checkbox-content">
                        ${purpose.description ? `<p class="cm-list-description">${purpose.description}</p>` : ''}
                        ${serviceHtml ?? ''}
                    </div>
                </div>
            </div>
        `;
        let html = '';
        let usedPurposes = this.manager.getUsedPurposes();
        for (const purpose of usedPurposes) {
            html += '<div class="purpose-group">';
            let serviceHtml = '';
            if (!purpose.hideServices) {
                for (const service of this.manager.services) {
                    if (service.purpose == purpose.name && (!service.contextualConsentOnly || this.manager.getConsent(service.name))) {
                        serviceHtml += '<div class="service">' + Template({ purpose: service, parent: service.purpose, translate: this.manager.translations }) + '</div>';
                    }
                }
            }
            html += Template({ purpose: purpose, parent: '', serviceHtml: serviceHtml, translate: this.manager.translations });

            html += '</div>';
        }
        return html;
    }
    isDisplayed() {
        if (!localStorage.getItem(this.consentHideStorageName)) {
            this.elements.get('html').classList.add('ig-consent-show');
        }
    }
    show() {
        this.elements.get('html').classList.add('ig-consent-show');
    }
    showSettings() {
        this.elements.get('html').classList.add('ig-consent-show');
        this.settings();
    }
    hideAndSave() {
        this.manager.saveAndApplyConsents();
        localStorage.setItem(this.consentHideStorageName, true);
        this.hide();
    }
    hide() {
        this.elements.get('html').classList.remove('ig-consent-show');
    }
    setElements() {
        this.elements.set('html', document.querySelector('html'));
        this.elements.set('outer-container', document.querySelector('.ig-consent-container'));
        this.elements.set('inner-container', document.querySelector('.ig-consent-container-inner'));
        this.elements.set('settings-container-list', document.querySelector('.settings-container-list'));

    }
    setButtons() {
        document.querySelectorAll('.ig-consent-button-save')?.forEach((el, i) => { el.addEventListener('click', () => { this.save() }) });
        document.querySelectorAll('.ig-consent-button-all')?.forEach((el, i) => { el.addEventListener('click', () => { this.all() }) });
        document.querySelectorAll('.ig-consent-button-needed')?.forEach((el, i) => {
            el.addEventListener('click', () => { this.needed() })
        });
        document.querySelectorAll('.ig-consent-button-settings')?.forEach((el, i) => {
            el.addEventListener('click', () => {
                this.settings();
            })
        });
        document.querySelectorAll('.ig-consent-button-close')?.forEach((el, i) => {
            el.addEventListener('click', () => {
                if (this.manager.confirmed) {
                    this.hide();
                } else {
                    this.close();
                }
            })
        });
    }
    update(consents) {
        this.updateServiceBoxes();
    }

    updatePurposeBoxes() {
        // check if all checkboxes of a purpuse are checked
        let allChecked = [];
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.dataset.parent) {
                let purpose = checkbox.dataset.parent;
                if (typeof allChecked[purpose] === 'undefined' || allChecked[purpose]) {
                    allChecked[purpose] = checkbox.checked;
                }
            }
        });

        this.elements.get('checkboxes-purpose').forEach(checkbox => {
            let consent = checkbox.value;
            if (allChecked[consent]) {
                checkbox.checked = true;
            } else {
                checkbox.checked = false;
            }
        });
    }
    setServicesByPurpose(parent, value) {
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.dataset.parent == parent) {
                if (checkbox.checked != value) {
                    this.manager.updateConsent(checkbox.value, value);
                }
                checkbox.checked = value;
            }
        });
    }

    // update services from consent manager
    updateServiceBoxes() {
        let checkboxes = this.elements.get('checkboxes-service');
        if (typeof checkboxes == 'undefined') {
            return;
        }
        checkboxes.forEach(checkbox => {
            let consent = checkbox.value;
            if (this.manager.getConsent(consent)) {
                checkbox.checked = true;
            } else {
                checkbox.checked = false;
            }
        });

        this.updatePurposeBoxes();
    }
    save() {
        this.elements.get('checkboxes-service').forEach(checkbox => {
            if (checkbox.checked) {
                this.manager.updateConsent(checkbox.value, true);
            } else {
                this.manager.updateConsent(checkbox.value, false);
            }
        });
        this.hideAndSave();
    }
    close() {
        this.elements.get('inner-container').classList.remove('settings-show');
    }
    needed() {
        this.hideAndSave();
    }
    all() {
        this.manager.changeAll(true);
        this.hideAndSave();
    }
    settings() {
        let settingsContainer = this.elements.get('settings-container-list');
        let content = this.getTemplate();
        if (!settingsContainer.innerHTML) {
            settingsContainer.innerHTML = content;
            this.elements.set('checkboxes-purpose', document.querySelectorAll('.ig-consent-container-inner .purpose .purpose-header input[type="checkbox"]:not(:disabled)'));
            this.elements.set('checkboxes-service', document.querySelectorAll('.ig-consent-container-inner .service input[type="checkbox"]:not(:disabled)'));
            this.elements.set('checkboxes-header', document.querySelectorAll('.ig-consent-container-inner .purpose .purpose-header.checkbox-headline .headline-click'));
            // add checkbox click events
            this.elements.get('checkboxes-purpose').forEach((checkbox) => {
                checkbox.addEventListener('click', () => {
                    this.setServicesByPurpose(checkbox.value, checkbox.checked);
                });
            });
            this.elements.get('checkboxes-service').forEach((checkbox) => {
                checkbox.addEventListener('click', () => {
                    this.manager.updateConsent(checkbox.value, checkbox.checked);
                    this.updatePurposeBoxes();
                    //this.updateServiceBoxes();
                });
            });
            // accordion
            this.elements.get('checkboxes-header').forEach((header) => {
                header.addEventListener('click', () => {
                    header.closest('.checkbox-headline').classList.toggle('is-open');
                });
            });


        }
        this.updateServiceBoxes();
        this.elements.get('inner-container').classList.add('settings-show');
    }
}

const igConsentUi = new IgConsentUi();
if (typeof window.igJs !== 'undefined') {
    window.igJs.add('igConsentUi', igConsentUi);
}


// cookie settings link for menus
var cookieSettings = document.querySelector('[href="#cookie-settings"]');
// fallback v10
if (cookieSettings === null) {
    cookieSettings = document.querySelector('[href="' + window.location.origin + '/#cookie-settings"]');
}
cookieSettings?.addEventListener('click', (e) => {
    e.preventDefault();
    igConsentUi.showSettings();
});
